/* zprof_off.p
 *
 * simple stand-alone non-persistent profiler shutdown
 *
 * bug in oe12.2.3 results in no profiling data, must upgrade to 12.2.4 or later
 *  https://knowledgebase.progress.com/articles/Knowledge/profiler-system-handle-write-data-does-not-write-any-output/
 *
 * run lib/zprof_off.p.
 *
 */

define new global shared variable profilerStart        as datetime  no-undo.
define new global shared variable profilerDescription  as character no-undo.
define new global shared variable profilerBaseName     as character no-undo.
define new global shared variable dbgMode              as integer   no-undo initial 3.

define variable profilerOutfile as character format "x(50)" no-undo.

if profiler:enabled <> yes then							/* not enabled	*/
  do:
    if dbgMode >= 5 then message now "profiler was not enabled".
    return.
  end.

/* append the date and time to the output file name so that we can easily keep a history
 */

profilerOutFile =
  session:temp-directory +
  profilerBaseName +
  substitute(
    ".&1.&2.&3.&4",
    year( today ),
    string( month( today ), "99" ),
    string( day( today ), "99" ),
    replace( string( time, "hh:mm:ss" ), ":", "." )
  )
.

assign
  profiler:description = profilerDescription + " [" + string( absolute( interval( profilerStart, now, "seconds" )), "hh:mm:ss" ) + "]"
  profiler:file-name   = profilerOutFile + ".prf"
.

profiler:profiling = no.

profiler:enabled = no.				/* use either "enabled = no" or "write-data()" but not both	*/

/*
 * enabled = no causes write-data() to execute, if you do both the 2nd overwrites
 * the first but since nothing happens in between the resulting file is empty
 *
 * profiler:write-data().
 * profiler:file-name = "empty".		/* prevent a null file from overwriting the real output		*/
 *
 */

if dbgMode >= 5 then
  message now "writing profiler [" + profiler:description + "] data:" profiler:file-name "has been written.".

return.
