#!/bin/sh
#

. #ENVPT3#/bin/protopenv

TMPDIR=${TMPDIR-/tmp}
LOGDIR=${LOGDIR-/tmp}

cd ${PROTOP}

if [ ! -d ${TMPDIR} ]
then
        mkdir ${TMPDIR}
fi

if [ ! -d ${LOGDIR} ]
then
	mkdir ${LOGDIR}
fi

errmsg()
{
	echo ""
	echo "${1}"
	echo ""
}

# FRNAME is a "friendly name" in etc/dblist.cfg

FRNAME=${1}

if [ -z "${FRNAME}" ]
then
	errmsg "Please at least specify a friendly name in etc/dblist.cfg."
	exit 1
fi

# Check etc/dblist.cfg for a matching "friendly name"
#

if [ -f ${PROTOP}/etc/dblist.cfg ]			# search using "pipe" as the delimiter
then
	XDB=$(grep "^${FRNAME}|" ${PROTOP}/etc/dblist.cfg | awk -F"|" '{print $2}' 2>/dev/null)
fi

# if XDB has a value then we found a friendly name and the path is in XDB #

if [ -z "${XDB}" ]					# no friendly name?
then
	errmsg "Friendly name ${FRNAME} does not exist."
	exit 1
fi

# customize LGARCDIR naming convention if necessary
# i.e.
# LGARCDIR=/db/${FRNAME}/logs

if [ ! -d ${LGARCDIR} ]
then
        errmsg "Creating ${LGARCDIR}"                
        mkdir ${LGARCDIR}
fi

if [ ! -d ${LGARCDIR} ]
then
        errmsg "Could not find or create ${LGARCDIR}"
        exit 1
fi

# the main event!
#

{

	echo `date` "truncating .lg file" ${FRNAME} ${XDB}

	if [ -d  "${LGARCDIR}" ]
	then
		# DT=`date +%m.%d`
		DT=`date +%W`
		cp ${XDB}.lg  ${LGARCDIR}/${FRNAME}.lg.${DT}
		chmod 666 ${XDB}.lic
		cp ${XDB}.lic ${LGARCDIR}/${FRNAME}.lic.${DT}
		> ${XDB}.lic
	fi

	${DLC}/bin/prolog ${XDB} -online

} >> ${LOGDIR}/prolog.${FRNAME}.log 2> ${TMPDIR}/prolog.${FRNAME}.err
