/* getfile.p
 *
 * simple program for executing a GET (and retreiving a file)
 *
 * pro -p util/getfile.p -param "host|path|outputfilename"
 *  or
 * bpro -p util/getfile.p -param "host|path|outputfilename" > logFile
 *
 * outputfilename is optional -- use it to fetch files, i.e.
 *
 *   pro -p util/getfile.p -param "demo.wss.com|/pt3upd/pt3.1415x-src.tar.gz|pt3.tar.gz"
 *
 */

define new global shared variable dbgMode as integer no-undo.

define variable host     as character no-undo.
define variable port     as character no-undo.
define variable path     as character no-undo.
define variable proxy    as character no-undo.
define variable creds    as character no-undo.
define variable options  as character no-undo.
define variable fName    as character no-undo.
define variable xtraHdrs as character no-undo.

define variable result   as longchar  no-undo.
define variable xrspHdr  as character no-undo.
define variable xstatus  as character no-undo.
define variable fLen     as integer   no-undo.


run util/restlib.p persistent.

result = "".
fix-codepage( result ) = 'utf-8'.			/* fix-codepage() can only be run when the argument is empty!   */

host = os-getenv( "PTHOST" ).
if host = ? then host = "".

port = os-getenv( "PORTALPORT" ).
if port = ? then port = "".

options = os-getenv( "PORTALOPTIONS" ).
if options = ? then options = "".

proxy = os-getenv( "PROXY" ).
if proxy = ? then proxy = "".

if num-entries( session:parameter, "|" ) < 2 then
  do:
    message 'usage: pro -p util/getfile.p -param "host|path[|fileName]"'.
    quit.
  end.

if num-entries( session:parameter, "|" ) >= 1 then host  = entry( 1, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 2 then path  = entry( 2, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 3 then fName = entry( 3, session:parameter, "|" ).

if num-entries( session:parameter, "|" ) >= 4 then port     = entry( 4, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 5 then proxy    = entry( 5, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 6 then creds    = entry( 6, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 7 then options  = entry( 7, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 8 then xtraHdrs = entry( 8, session:parameter, "|" ).

message now "getfile:" session:parameter.

dbgMode = 3.

if host = "" then host = "demo.wss.com".
if port = "" then port = "80".

message now substitute( "getURL: host=&1 port=&2 path=&3 creds=&4 options=&5 proxy=&6 fname=&7",host, port, path, creds, options, proxy, xtraHdrs, fName ).

run getURL( host, port, path, creds, options, proxy, xtraHdrs, fName, output result, output xrspHdr ).

xstatus = entry( 1, xrspHdr, "~n" ).

message now "status:" xstatus.

if xStatus matches "HTTP* 2* OK" and fName <> "" and fName <> ? then
  do:
    file-info:file-name = fName.
    fLen = file-info:file-size.
    message now "downloaded" fLen "bytes to file:" fName.
  end.

quit.
